@@ -221,7 +221,6 @@ bool Bind2Backend::startTransaction(const DNSName& qname, int id)
     int fd = mkstemp(&d_transaction_tmpname.at(0));
     if (fd == -1) {
       throw DBException("Unable to create a unique temporary zonefile '" + d_transaction_tmpname + "': " + stringerror());
-      return false;
     }
 
     d_of = std::make_unique<ofstream>(d_transaction_tmpname);
@@ -319,7 +318,7 @@ bool Bind2Backend::feedRecord(const DNSResourceRecord& rr, const DNSName& ordern
   return true;
 }
 
-void Bind2Backend::getUpdatedMasters(vector<DomainInfo>* changedDomains)
+void Bind2Backend::getUpdatedMasters(vector<DomainInfo>& changedDomains, std::unordered_set<DNSName>& catalogs, CatalogHashMap& catalogHashes)
 {
   vector<DomainInfo> consider;
   {
@@ -357,7 +356,7 @@ void Bind2Backend::getUpdatedMasters(vector<DomainInfo>* changedDomains)
       }
       if (di.notified_serial) { // don't do notification storm on startup
         di.serial = soadata.serial;
-        changedDomains->push_back(std::move(di));
+        changedDomains.push_back(std::move(di));
       }
     }
   }
@@ -1207,17 +1206,9 @@ void Bind2Backend::lookup(const QType& qtype, const DNSName& qname, int zoneId,
   auto& hashedidx = boost::multi_index::get<UnorderedNameTag>(*d_handle.d_records);
   auto range = hashedidx.equal_range(d_handle.qname);
 
-  if (range.first == range.second) {
-    d_handle.d_list = false;
-    d_handle.d_iter = d_handle.d_end_iter = range.first;
-    return;
-  }
-  else {
-    d_handle.d_iter = range.first;
-    d_handle.d_end_iter = range.second;
-  }
-
   d_handle.d_list = false;
+  d_handle.d_iter = range.first;
+  d_handle.d_end_iter = range.second;
 }
 
 Bind2Backend::handle::handle()
