@@ -283,7 +283,7 @@ unique_ptr<GeoIPInterface> GeoIPInterface::makeMMDBInterface(const string& fname
   const auto& opt_lang = opts.find("language");
   if (opt_lang != opts.end())
     language = opt_lang->second;
-  return unique_ptr<GeoIPInterface>(new GeoIPInterfaceMMDB(fname, mode, language));
+  return std::make_unique<GeoIPInterfaceMMDB>(fname, mode, language);
 }
 
 #else
