@@ -31,8 +31,7 @@
 #include "dnspcap.hh"
 #include "dnsparser.hh"
 #include "dnsname.hh"
-#include <boost/tuple/tuple.hpp>
-#include <boost/tuple/tuple_comparison.hpp>
+#include <boost/format.hpp>
 #include <map>
 #include <set>
 #include <fstream>
