@@ -36,7 +36,7 @@
 #include "misc.hh"
 #include <vector>
 #include <sstream>
-#include <errno.h>
+#include <cerrno>
 #include <cstring>
 #include <iostream>
 #include <sys/types.h>
@@ -45,12 +45,14 @@
 #include <poll.h>
 #include <iomanip>
 #include <netinet/tcp.h>
+#include <optional>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include "pdnsexception.hh"
 #include <sys/types.h>
 #include <boost/algorithm/string.hpp>
+#include <boost/format.hpp>
 #include "iputils.hh"
 #include "dnsparser.hh"
 #include <sys/types.h>
@@ -197,6 +199,32 @@ size_t writen2WithTimeout(int fd, const void * buffer, size_t len, const struct
   return len;
 }
 
+auto pdns::getMessageFromErrno(const int errnum) -> std::string
+{
+  const size_t errLen = 2048;
+  std::string errMsgData{};
+  errMsgData.resize(errLen);
+
+  const char* errMsg = nullptr;
+#ifdef STRERROR_R_CHAR_P
+  errMsg = strerror_r(errnum, errMsgData.data(), errMsgData.length());
+#else
+  // This can fail, and when it does, it sets errno. We ignore that and
+  // set our own error message instead.
+  int res = strerror_r(errnum, errMsgData.data(), errMsgData.length());
+  errMsg = errMsgData.c_str();
+  if (res != 0) {
+    errMsg = "Unknown (the exact error could not be retrieved)";
+  }
+#endif
+
+  // We make a copy here because `strerror_r()` might return a static
+  // immutable buffer for an error message. The copy shouldn't be
+  // critical though, we're on the bailout/error-handling path anyways.
+  std::string message{errMsg};
+  return message;
+}
+
 string nowTime()
 {
   time_t now = time(nullptr);
@@ -204,8 +232,10 @@ string nowTime()
   localtime_r(&now, &tm);
   char buffer[30];
   // YYYY-mm-dd HH:MM:SS TZOFF
-  strftime(buffer, sizeof(buffer), "%F %T %z", &tm);
-  buffer[sizeof(buffer)-1] = '\0';
+  size_t ret = strftime(buffer, sizeof(buffer), "%F %T %z", &tm);
+  if (ret == 0) {
+    buffer[0] = '\0';
+  }
   return string(buffer);
 }
 
@@ -280,29 +310,32 @@ bool stripDomainSuffix(string *qname, const string &domain)
   return true;
 }
 
-static void parseService4(const string &descr, ServiceTuple &st)
+static void parseService4(const string& descr, ServiceTuple& st)
 {
-  vector<string>parts;
-  stringtok(parts,descr,":");
-  if(parts.empty())
-    throw PDNSException("Unable to parse '"+descr+"' as a service");
-  st.host=parts[0];
-  if(parts.size()>1)
-    st.port=pdns_stou(parts[1]);
+  vector<string> parts;
+  stringtok(parts, descr, ":");
+  if (parts.empty()) {
+    throw PDNSException("Unable to parse '" + descr + "' as a service");
+  }
+  st.host = parts[0];
+  if (parts.size() > 1) {
+    pdns::checked_stoi_into(st.port, parts[1]);
+  }
 }
 
-static void parseService6(const string &descr, ServiceTuple &st)
+static void parseService6(const string& descr, ServiceTuple& st)
 {
-  string::size_type pos=descr.find(']');
-  if(pos == string::npos)
-    throw PDNSException("Unable to parse '"+descr+"' as an IPv6 service");
+  string::size_type pos = descr.find(']');
+  if (pos == string::npos) {
+    throw PDNSException("Unable to parse '" + descr + "' as an IPv6 service");
+  }
 
-  st.host=descr.substr(1, pos-1);
-  if(pos + 2 < descr.length())
-    st.port=pdns_stou(descr.substr(pos+2));
+  st.host = descr.substr(1, pos - 1);
+  if (pos + 2 < descr.length()) {
+    pdns::checked_stoi_into(st.port, descr.substr(pos + 2));
+  }
 }
 
-
 void parseService(const string &descr, ServiceTuple &st)
 {
   if(descr.empty())
@@ -582,18 +615,14 @@ string makeBytesFromHex(const string &in) {
     throw std::range_error("odd number of bytes in hex string");
   }
   string ret;
-  ret.reserve(in.size() / 2);
-
+  ret.reserve(in.size());
   unsigned int num;
-  for (size_t i = 0; i < in.size(); i += 2) {
-    const auto numStr = in.substr(i, 2);
+  for (size_t i = 0; i < in.size(); i+=2) {
+    string numStr = in.substr(i, 2);
     num = 0;
-    if (sscanf(numStr.c_str(), "%02x", &num) != 1) {
-      throw std::range_error("Invalid value while parsing the hex string '" + in + "'");
-    }
-    ret.push_back(static_cast<uint8_t>(num));
+    sscanf(numStr.c_str(), "%02x", &num);
+    ret.push_back((uint8_t)num);
   }
-
   return ret;
 }
 
@@ -675,43 +704,47 @@ string stripDot(const string& dom)
   return dom.substr(0,dom.size()-1);
 }
 
-
-
 int makeIPv6sockaddr(const std::string& addr, struct sockaddr_in6* ret)
 {
-  if(addr.empty())
+  if (addr.empty()) {
     return -1;
+  }
+
   string ourAddr(addr);
-  bool portSet = false;
-  unsigned int port;
-  if(addr[0]=='[') { // [::]:53 style address
+  std::optional<uint16_t> port = std::nullopt;
+
+  if (addr[0] == '[') { // [::]:53 style address
     string::size_type pos = addr.find(']');
-    if(pos == string::npos)
+    if (pos == string::npos) {
       return -1;
-    ourAddr.assign(addr.c_str() + 1, pos-1);
+    }
+
+    ourAddr.assign(addr.c_str() + 1, pos - 1);
     if (pos + 1 != addr.size()) { // complete after ], no port specified
-      if (pos + 2 > addr.size() || addr[pos+1]!=':')
+      if (pos + 2 > addr.size() || addr[pos + 1] != ':') {
         return -1;
-      try {
-        port = pdns_stou(addr.substr(pos+2));
-        portSet = true;
       }
-      catch(const std::out_of_range&) {
+
+      try {
+        auto tmpPort = pdns::checked_stoi<uint16_t>(addr.substr(pos + 2));
+        port = std::make_optional(tmpPort);
+      }
+      catch (const std::out_of_range&) {
         return -1;
       }
     }
   }
-  ret->sin6_scope_id=0;
-  ret->sin6_family=AF_INET6;
 
-  if(inet_pton(AF_INET6, ourAddr.c_str(), (void*)&ret->sin6_addr) != 1) {
-    struct addrinfo* res;
-    struct addrinfo hints;
-    memset(&hints, 0, sizeof(hints));
+  ret->sin6_scope_id = 0;
+  ret->sin6_family = AF_INET6;
 
-    hints.ai_family = AF_INET6;
+  if (inet_pton(AF_INET6, ourAddr.c_str(), (void*)&ret->sin6_addr) != 1) {
+    struct addrinfo hints{};
+    std::memset(&hints, 0, sizeof(struct addrinfo));
     hints.ai_flags = AI_NUMERICHOST;
+    hints.ai_family = AF_INET6;
 
+    struct addrinfo* res = nullptr;
     // getaddrinfo has anomalous return codes, anything nonzero is an error, positive or negative
     if (getaddrinfo(ourAddr.c_str(), nullptr, &hints, &res) != 0) {
       return -1;
@@ -721,11 +754,8 @@ int makeIPv6sockaddr(const std::string& addr, struct sockaddr_in6* ret)
     freeaddrinfo(res);
   }
 
-  if(portSet) {
-    if(port > 65535)
-      return -1;
-
-    ret->sin6_port = htons(port);
+  if (port.has_value()) {
+    ret->sin6_port = htons(*port);
   }
 
   return 0;
@@ -798,12 +828,10 @@ bool readFileIfThere(const char* fname, std::string* line)
 {
   line->clear();
   auto fp = std::unique_ptr<FILE, int(*)(FILE*)>(fopen(fname, "r"), fclose);
-  if(!fp)
+  if (!fp) {
     return false;
-  stringfgets(fp.get(), *line);
-  fp.reset();
-
-  return true;
+  }
+  return stringfgets(fp.get(), *line);
 }
 
 Regex::Regex(const string &expr)
@@ -915,120 +943,6 @@ void setFilenumLimit(unsigned int lim)
     unixDie("Setting number of available file descriptors");
 }
 
-#define burtlemix(a,b,c) \
-{ \
-  a -= b; a -= c; a ^= (c>>13); \
-  b -= c; b -= a; b ^= (a<<8); \
-  c -= a; c -= b; c ^= (b>>13); \
-  a -= b; a -= c; a ^= (c>>12);  \
-  b -= c; b -= a; b ^= (a<<16); \
-  c -= a; c -= b; c ^= (b>>5); \
-  a -= b; a -= c; a ^= (c>>3);  \
-  b -= c; b -= a; b ^= (a<<10); \
-  c -= a; c -= b; c ^= (b>>15); \
-}
-
-uint32_t burtle(const unsigned char* k, uint32_t length, uint32_t initval)
-{
-  uint32_t a,b,c,len;
-
-   /* Set up the internal state */
-  len = length;
-  a = b = 0x9e3779b9;  /* the golden ratio; an arbitrary value */
-  c = initval;         /* the previous hash value */
-
-  /*---------------------------------------- handle most of the key */
-  while (len >= 12) {
-    a += (k[0] +((uint32_t)k[1]<<8) +((uint32_t)k[2]<<16) +((uint32_t)k[3]<<24));
-    b += (k[4] +((uint32_t)k[5]<<8) +((uint32_t)k[6]<<16) +((uint32_t)k[7]<<24));
-    c += (k[8] +((uint32_t)k[9]<<8) +((uint32_t)k[10]<<16)+((uint32_t)k[11]<<24));
-    burtlemix(a,b,c);
-    k += 12; len -= 12;
-  }
-
-  /*------------------------------------- handle the last 11 bytes */
-  c += length;
-  switch(len) {             /* all the case statements fall through */
-  case 11: c+=((uint32_t)k[10]<<24);
-    /* fall-through */
-  case 10: c+=((uint32_t)k[9]<<16);
-    /* fall-through */
-  case 9 : c+=((uint32_t)k[8]<<8);
-    /* the first byte of c is reserved for the length */
-    /* fall-through */
-  case 8 : b+=((uint32_t)k[7]<<24);
-    /* fall-through */
-  case 7 : b+=((uint32_t)k[6]<<16);
-    /* fall-through */
-  case 6 : b+=((uint32_t)k[5]<<8);
-    /* fall-through */
-  case 5 : b+=k[4];
-    /* fall-through */
-  case 4 : a+=((uint32_t)k[3]<<24);
-    /* fall-through */
-  case 3 : a+=((uint32_t)k[2]<<16);
-    /* fall-through */
-  case 2 : a+=((uint32_t)k[1]<<8);
-    /* fall-through */
-  case 1 : a+=k[0];
-    /* case 0: nothing left to add */
-  }
-  burtlemix(a,b,c);
-  /*-------------------------------------------- report the result */
-  return c;
-}
-
-uint32_t burtleCI(const unsigned char* k, uint32_t length, uint32_t initval)
-{
-  uint32_t a,b,c,len;
-
-   /* Set up the internal state */
-  len = length;
-  a = b = 0x9e3779b9;  /* the golden ratio; an arbitrary value */
-  c = initval;         /* the previous hash value */
-
-  /*---------------------------------------- handle most of the key */
-  while (len >= 12) {
-    a += (dns_tolower(k[0]) +((uint32_t)dns_tolower(k[1])<<8) +((uint32_t)dns_tolower(k[2])<<16) +((uint32_t)dns_tolower(k[3])<<24));
-    b += (dns_tolower(k[4]) +((uint32_t)dns_tolower(k[5])<<8) +((uint32_t)dns_tolower(k[6])<<16) +((uint32_t)dns_tolower(k[7])<<24));
-    c += (dns_tolower(k[8]) +((uint32_t)dns_tolower(k[9])<<8) +((uint32_t)dns_tolower(k[10])<<16)+((uint32_t)dns_tolower(k[11])<<24));
-    burtlemix(a,b,c);
-    k += 12; len -= 12;
-  }
-
-  /*------------------------------------- handle the last 11 bytes */
-  c += length;
-  switch(len) {             /* all the case statements fall through */
-  case 11: c+=((uint32_t)dns_tolower(k[10])<<24);
-    /* fall-through */
-  case 10: c+=((uint32_t)dns_tolower(k[9])<<16);
-    /* fall-through */
-  case 9 : c+=((uint32_t)dns_tolower(k[8])<<8);
-    /* the first byte of c is reserved for the length */
-    /* fall-through */
-  case 8 : b+=((uint32_t)dns_tolower(k[7])<<24);
-    /* fall-through */
-  case 7 : b+=((uint32_t)dns_tolower(k[6])<<16);
-    /* fall-through */
-  case 6 : b+=((uint32_t)dns_tolower(k[5])<<8);
-    /* fall-through */
-  case 5 : b+=dns_tolower(k[4]);
-    /* fall-through */
-  case 4 : a+=((uint32_t)dns_tolower(k[3])<<24);
-    /* fall-through */
-  case 3 : a+=((uint32_t)dns_tolower(k[2])<<16);
-    /* fall-through */
-  case 2 : a+=((uint32_t)dns_tolower(k[1])<<8);
-    /* fall-through */
-  case 1 : a+=dns_tolower(k[0]);
-    /* case 0: nothing left to add */
-  }
-  burtlemix(a,b,c);
-  /*-------------------------------------------- report the result */
-  return c;
-}
-
-
 bool setSocketTimestamps(int fd)
 {
 #ifdef SO_TIMESTAMP
@@ -1115,31 +1029,126 @@ bool setCloseOnExec(int sock)
   return true;
 }
 
+#ifdef __linux__
+#include <linux/rtnetlink.h>
+
+int getMACAddress(const ComboAddress& ca, char* dest, size_t destLen)
+{
+  struct {
+    struct nlmsghdr headermsg;
+    struct ndmsg neighbormsg;
+  } request;
+
+  std::array<char, 8192> buffer;
+
+  auto sock = FDWrapper(socket(AF_NETLINK, SOCK_RAW|SOCK_CLOEXEC, NETLINK_ROUTE));
+  if (sock.getHandle() == -1) {
+    return errno;
+  }
+
+  memset(&request, 0, sizeof(request));
+  request.headermsg.nlmsg_len = NLMSG_LENGTH(sizeof(struct ndmsg));
+  request.headermsg.nlmsg_flags = NLM_F_REQUEST | NLM_F_DUMP;
+  request.headermsg.nlmsg_type = RTM_GETNEIGH;
+  request.neighbormsg.ndm_family = ca.sin4.sin_family;
+
+  while (true) {
+    ssize_t sent = send(sock.getHandle(), &request, sizeof(request), 0);
+    if (sent == -1) {
+      if (errno == EINTR) {
+        continue;
+      }
+      return errno;
+    }
+    else if (static_cast<size_t>(sent) != sizeof(request)) {
+      return EIO;
+    }
+    break;
+  }
+
+  bool done = false;
+  bool foundIP = false;
+  bool foundMAC = false;
+  do {
+    ssize_t got = recv(sock.getHandle(), buffer.data(), buffer.size(), 0);
+
+    if (got < 0) {
+      if (errno == EINTR) {
+        continue;
+      }
+      return errno;
+    }
+
+    size_t remaining = static_cast<size_t>(got);
+    for (struct nlmsghdr* nlmsgheader = reinterpret_cast<struct nlmsghdr*>(buffer.data());
+         done == false && NLMSG_OK (nlmsgheader, remaining);
+         nlmsgheader = reinterpret_cast<struct nlmsghdr*>(NLMSG_NEXT(nlmsgheader, remaining))) {
+
+      if (nlmsgheader->nlmsg_type == NLMSG_DONE) {
+        done = true;
+        break;
+      }
+
+      auto nd = reinterpret_cast<struct ndmsg*>(NLMSG_DATA(nlmsgheader));
+      auto rtatp = reinterpret_cast<struct rtattr*>(reinterpret_cast<char*>(nd) + NLMSG_ALIGN(sizeof(struct ndmsg)));
+      int rtattrlen = nlmsgheader->nlmsg_len - NLMSG_LENGTH(sizeof(struct ndmsg));
+
+      if (nd->ndm_family != ca.sin4.sin_family) {
+        continue;
+      }
+
+      if (ca.sin4.sin_family == AF_INET6 && ca.sin6.sin6_scope_id != 0 && static_cast<int32_t>(ca.sin6.sin6_scope_id) != nd->ndm_ifindex) {
+        continue;
+      }
+
+      for (; done == false && RTA_OK(rtatp, rtattrlen); rtatp = RTA_NEXT(rtatp, rtattrlen)) {
+        if (rtatp->rta_type == NDA_DST){
+          if (nd->ndm_family == AF_INET) {
+            auto inp = reinterpret_cast<struct in_addr*>(RTA_DATA(rtatp));
+            if (inp->s_addr == ca.sin4.sin_addr.s_addr) {
+              foundIP = true;
+            }
+          }
+          else if (nd->ndm_family == AF_INET6) {
+            auto inp = reinterpret_cast<struct in6_addr *>(RTA_DATA(rtatp));
+            if (memcmp(inp->s6_addr, ca.sin6.sin6_addr.s6_addr, sizeof(ca.sin6.sin6_addr.s6_addr)) == 0) {
+              foundIP = true;
+            }
+          }
+        }
+        else if (rtatp->rta_type == NDA_LLADDR) {
+          if (foundIP) {
+            size_t addrLen = rtatp->rta_len - sizeof(struct rtattr);
+            if (addrLen > destLen) {
+              return ENOBUFS;
+            }
+            memcpy(dest, reinterpret_cast<const char*>(rtatp) + sizeof(struct rtattr), addrLen);
+            foundMAC = true;
+            done = true;
+            break;
+          }
+        }
+      }
+    }
+  }
+  while (done == false);
+
+  return foundMAC ? 0 : ENOENT;
+}
+#else
+int getMACAddress(const ComboAddress& ca, char* dest, size_t len)
+{
+  return ENOENT;
+}
+#endif /* __linux__ */
+
 string getMACAddress(const ComboAddress& ca)
 {
   string ret;
-#ifdef __linux__
-  ifstream ifs("/proc/net/arp");
-  if(!ifs)
-    return ret;
-  string line;
-  string match=ca.toString()+' ';
-  while(getline(ifs, line)) {
-    if(boost::starts_with(line, match)) {
-      vector<string> parts;
-      stringtok(parts, line, " \n\t\r");
-      if(parts.size() < 4)
-        return ret;
-      unsigned int tmp[6];
-      if (sscanf(parts[3].c_str(), "%02x:%02x:%02x:%02x:%02x:%02x", tmp, tmp+1, tmp+2, tmp+3, tmp+4, tmp+5) != 6) {
-        return ret;
-      }
-      for(unsigned int i : tmp)
-        ret.append(1, (char)i);
-      return ret;
-    }
+  char tmp[6];
+  if (getMACAddress(ca, tmp, sizeof(tmp)) == 0) {
+    ret.append(tmp, sizeof(tmp));
   }
-#endif
   return ret;
 }
 
@@ -1340,7 +1349,7 @@ uint64_t getOpenFileDescriptors(const std::string&)
 {
 #ifdef __linux__
   DIR* dirhdl=opendir(("/proc/"+std::to_string(getpid())+"/fd/").c_str());
-  if(!dirhdl) 
+  if(!dirhdl)
     return 0;
 
   struct dirent *entry;
@@ -1348,7 +1357,7 @@ uint64_t getOpenFileDescriptors(const std::string&)
   while((entry = readdir(dirhdl))) {
     uint32_t num;
     try {
-      num = pdns_stou(entry->d_name);
+      pdns::checked_stoi_into(num, entry->d_name);
     } catch (...) {
       continue; // was not a number.
     }
@@ -1377,7 +1386,7 @@ uint64_t getRealMemoryUsage(const std::string&)
   ifs >> size >> resident >> shared >> text >> lib >> data;
 
   // We used to use "data" here, but it proves unreliable and even is marked "broken"
-  // in https://www.kernel.org/doc/html/latest/filesystems/proc.html 
+  // in https://www.kernel.org/doc/html/latest/filesystems/proc.html
   return resident * getpagesize();
 #else
   struct rusage ru;
@@ -1426,7 +1435,7 @@ double DiffTime(const struct timespec& first, const struct timespec& second)
 {
   int seconds=second.tv_sec - first.tv_sec;
   int nseconds=second.tv_nsec - first.tv_nsec;
-  
+
   if(nseconds < 0) {
     seconds-=1;
     nseconds+=1000000000;
@@ -1438,7 +1447,7 @@ double DiffTime(const struct timeval& first, const struct timeval& second)
 {
   int seconds=second.tv_sec - first.tv_sec;
   int useconds=second.tv_usec - first.tv_usec;
-  
+
   if(useconds < 0) {
     seconds-=1;
     useconds+=1000000;
@@ -1504,25 +1513,6 @@ gid_t strToGID(const string &str)
   return result;
 }
 
-unsigned int pdns_stou(const std::string& str, size_t * idx, int base)
-{
-  if (str.empty()) return 0; // compatibility
-  unsigned long result;
-  try {
-    result = std::stoul(str, idx, base);
-  }
-  catch(std::invalid_argument& e) {
-    throw std::invalid_argument(string(e.what()) + "; (invalid argument during std::stoul); data was \""+str+"\"");
-  }
-  catch(std::out_of_range& e) {
-    throw std::out_of_range(string(e.what()) + "; (out of range during std::stoul); data was \""+str+"\"");
-  }
-  if (result > std::numeric_limits<unsigned int>::max()) {
-    throw std::out_of_range("stoul returned result out of unsigned int range; data was \""+str+"\"");
-  }
-  return static_cast<unsigned int>(result);
-}
-
 bool isSettingThreadCPUAffinitySupported()
 {
 #ifdef HAVE_PTHREAD_SETAFFINITY_NP
