@@ -164,7 +164,7 @@ string NSECBitmap::getZoneRepresentation() const
   return ret;
 }
 
-void NSECRecordContent::report(void)
+void NSECRecordContent::report()
 {
   regist(1, 47, &make, &make, "NSEC");
 }
@@ -213,7 +213,7 @@ string NSECRecordContent::getZoneRepresentation(bool noDot) const
 
 ////// begin of NSEC3
 
-void NSEC3RecordContent::report(void)
+void NSEC3RecordContent::report()
 {
   regist(1, 50, &make, &make, "NSEC3");
 }
@@ -286,7 +286,7 @@ string NSEC3RecordContent::getZoneRepresentation(bool noDot) const
 }
 
 
-void NSEC3PARAMRecordContent::report(void)
+void NSEC3PARAMRecordContent::report()
 {
   regist(1, 51, &make, &make, "NSEC3PARAM");
   regist(254, 51, &make, &make, "NSEC3PARAM");
@@ -339,3 +339,59 @@ string NSEC3PARAMRecordContent::getZoneRepresentation(bool noDot) const
   return ret;
 }
 
+////// end of NSEC3
+
+////// begin of CSYNC
+
+void CSYNCRecordContent::report()
+{
+  regist(1, 62, &make, &make, "CSYNC");
+}
+
+std::shared_ptr<DNSRecordContent> CSYNCRecordContent::make(const string& content)
+{
+  return std::make_shared<CSYNCRecordContent>(content);
+}
+
+CSYNCRecordContent::CSYNCRecordContent(const string& content, const DNSName& zone)
+{
+  RecordTextReader rtr(content, zone);
+  rtr.xfr32BitInt(d_serial);
+  rtr.xfr16BitInt(d_flags);
+
+  while(!rtr.eof()) {
+    uint16_t type;
+    rtr.xfrType(type);
+    set(type);
+  }
+}
+
+void CSYNCRecordContent::toPacket(DNSPacketWriter& pw)
+{
+  pw.xfr32BitInt(d_serial);
+  pw.xfr16BitInt(d_flags);
+
+  d_bitmap.toPacket(pw);
+}
+
+std::shared_ptr<CSYNCRecordContent::DNSRecordContent> CSYNCRecordContent::make(const DNSRecord &dr, PacketReader& pr)
+{
+  auto ret=std::make_shared<CSYNCRecordContent>();
+  pr.xfr32BitInt(ret->d_serial);
+  pr.xfr16BitInt(ret->d_flags);
+
+  ret->d_bitmap.fromPacket(pr);
+  return ret;
+}
+
+string CSYNCRecordContent::getZoneRepresentation(bool noDot) const
+{
+  string ret;
+  RecordTextWriter rtw(ret);
+  rtw.xfr32BitInt(d_serial);
+  rtw.xfr16BitInt(d_flags);
+
+  return ret + d_bitmap.getZoneRepresentation();
+}
+
+////// end of CSYNC
