@@ -121,7 +121,7 @@ static void printReply(const string& reply, bool showflags, bool hidesoadetails,
   for (MOADNSParser::answers_t::const_iterator i = mdp.d_answers.begin();
        i != mdp.d_answers.end(); ++i) {
     cout << i->first.d_place - 1 << "\t" << i->first.d_name.toString() << "\t"
-         << nameForClass(i->first.d_class, i->first.d_type) << "\t"
+         << ttl(i->first.d_ttl) << "\t" << nameForClass(i->first.d_class, i->first.d_type) << "\t"
          << DNSRecordContent::NumberToType(i->first.d_type);
     if (dumpluaraw) {
       cout<<"\t"<< makeLuaString(i->first.d_content->serialize(DNSName(), true))<<endl;
@@ -132,7 +132,7 @@ static void printReply(const string& reply, bool showflags, bool hidesoadetails,
         string zoneRep = i->first.d_content->getZoneRepresentation();
         vector<string> parts;
         stringtok(parts, zoneRep);
-        cout << "\t" << ttl(i->first.d_ttl) << "\t" << parts[0] << " "
+        cout << "\t" << parts[0] << " "
              << parts[1] << " " << parts[2] << " " << parts[3]
              << " [expiry] [inception] [keytag] " << parts[7] << " ...\n";
         continue;
@@ -141,7 +141,7 @@ static void printReply(const string& reply, bool showflags, bool hidesoadetails,
         string zoneRep = i->first.d_content->getZoneRepresentation();
         vector<string> parts;
         stringtok(parts, zoneRep);
-        cout << "\t" << ttl(i->first.d_ttl) << "\t" << parts[0] << " [flags] "
+        cout << "\t" << parts[0] << " [flags] "
              << parts[2] << " " << parts[3] << " " << parts[4];
         for (vector<string>::iterator iter = parts.begin() + 5;
              iter != parts.end(); ++iter)
@@ -153,7 +153,7 @@ static void printReply(const string& reply, bool showflags, bool hidesoadetails,
         string zoneRep = i->first.d_content->getZoneRepresentation();
         vector<string> parts;
         stringtok(parts, zoneRep);
-        cout << "\t" << ttl(i->first.d_ttl) << "\t" << parts[0] << " "
+        cout << "\t" << parts[0] << " "
              << parts[1] << " " << parts[2] << " ...\n";
         continue;
       }
@@ -161,14 +161,13 @@ static void printReply(const string& reply, bool showflags, bool hidesoadetails,
         string zoneRep = i->first.d_content->getZoneRepresentation();
         vector<string> parts;
         stringtok(parts, zoneRep);
-        cout << "\t" << ttl(i->first.d_ttl) << "\t" << parts[0] << " "
+        cout << "\t" << parts[0] << " "
              << parts[1] << " [serial] " << parts[3] << " " << parts[4] << " "
              << parts[5] << " " << parts[6] << "\n";
         continue;
       }
     }
-    cout << "\t" << ttl(i->first.d_ttl) << "\t"
-         << i->first.d_content->getZoneRepresentation() << "\n";
+    cout << "\t" << i->first.d_content->getZoneRepresentation() << "\n";
   }
 
   EDNSOpts edo;
@@ -418,7 +417,7 @@ try {
     Socket sock(dest.sin4.sin_family, SOCK_STREAM);
     sock.setNonBlocking();
     setTCPNoDelay(sock.getHandle()); // disable NAGLE, which does not play nicely with delayed ACKs
-    TCPIOHandler handler(subjectName, sock.releaseHandle(), timeout, tlsCtx, time(nullptr));
+    TCPIOHandler handler(subjectName, false, sock.releaseHandle(), timeout, tlsCtx, time(nullptr));
     handler.connect(fastOpen, dest, timeout);
     // we are writing the proxyheader inside the TLS connection. Is that right?
     if (proxyheader.size() > 0 && handler.write(proxyheader.data(), proxyheader.size(), timeout) != proxyheader.size()) {
