@@ -24,6 +24,8 @@
 #endif
 
 #include <boost/tokenizer.hpp>
+#include <boost/format.hpp>
+
 #include "namespaces.hh"
 #include "ws-api.hh"
 #include "json.hh"
@@ -42,8 +44,6 @@
 
 using json11::Json;
 
-extern string s_programname;
-extern ResponseStats g_rs;
 #ifndef RECURSOR
 extern StatBag S;
 #endif
